//**********************************************************************
//COPYRIGHT NOTICE AND DISCLAIMER
//
//Copyright (c) Erik Meijering. All rights reserved.
//
//Permission to copy and use this software and accompanying
//documentation for educational, research, and not-for-profit
//purposes, without fee and without a signed licensing agreement,
//is hereby granted, provided that the above copyright notice, this
//paragraph and the following two paragraphs appear in all copies.
//The copyright holder is free to make upgraded or improved versions
//of the software available for a fee or commercially only.
//
//IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE TO ANY PARTY FOR
//DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
//OF ANY KIND WHATSOEVER, ARISING OUT OF THE USE OF THIS SOFTWARE
//AND ITS DOCUMENTATION, EVEN IF HE HAS BEEN ADVISED OF THE
//POSSIBILITY OF SUCH DAMAGE.
//
//THE COPYRIGHT HOLDER SPECIFICALLY DISCLAIMS ANY WARRANTIES,
//INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
//MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
//SOFTWARE AND ACCOMPANYING DOCUMENTATION IS PROVIDED "AS IS".
//THE COPYRIGHT HOLDER HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
//SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
//**********************************************************************
package neuronJgui;

import java.io.IOException;

public class BrowserControl {

	  /**
   * Display a file in the system browser.  If you want to display a
   * file, you must include the absolute path name.
   *
   * @param url the file's url (the url must start with either "http://"
or
   * "file://").
   */
  public static void displayURL(String url)
  {
      boolean windows = isWindowsPlatform();
      String cmd = null;
      try
      {
          if (windows)
          {
              // cmd = 'rundll32 url.dll,FileProtocolHandler http://...'
              cmd = "cmd /C " + WIN_PATH + " " + WIN_FLAG + " " + "\"" + url + "\"";
              System.out.println(cmd);
              Process p = Runtime.getRuntime().exec(cmd);
          }
          else
          {
              // Under Unix, Netscape has to be running for the "-remote"
              // command to work.  So, we try sending the command and
              // check for an exit value.  If the exit command is 0,
              // it worked, otherwise we need to start the browser.
              // cmd = 'netscape -remote openURL(http://www.javaworld.com)'
              cmd = UNIX_PATH + " " + UNIX_FLAG + "(" + url + ")";
              Process p = Runtime.getRuntime().exec(cmd);
              try
              {
                  // wait for exit code -- if it's 0, command worked,
                  // otherwise we need to start the browser up.
                  int exitCode = p.waitFor();
                  if (exitCode != 0)
                  {
                      // Command failed, start up the browser
                      // cmd = 'netscape http://www.javaworld.com'
                      cmd = UNIX_PATH + " "  + url;
                      p = Runtime.getRuntime().exec(cmd);
                  }
              }
              catch(InterruptedException x)
              {
                  System.err.println("Error bringing up browser, cmd='" +
                                     cmd + "'");
                  System.err.println("Caught: " + x);
              }
          }
      }
      catch(IOException x)
      {
          // couldn't exec browser
          System.err.println("Could not invoke browser, command=" + cmd);
          System.err.println("Caught: " + x);
      }
  }
  /**
   * Try to determine whether this application is running under Windows
   * or some other platform by examing the "os.name" property.
   *
   * @return true if this application is running under a Windows OS
   */
  public static boolean isWindowsPlatform()
  {
      String os = System.getProperty("os.name");
      if ( os != null && os.startsWith(WIN_ID))
          return true;
      else
          return false;

  }
  /**
   * Simple example.
   */
  public static void main(String[] args)
  {
      displayURL("http://www.javaworld.com");
  }
  // Used to identify the windows platform.
  private static final String WIN_ID = "Windows";
  // The default system browser under windows.
  private static final String WIN_PATH = "rundll32";
  // The flag to display a url.
  private static final String WIN_FLAG = "url.dll,FileProtocolHandler";
  // The default browser under unix.
  private static final String UNIX_PATH = "netscape";
  // The flag to display a url.
  private static final String UNIX_FLAG = "-remote openURL";
}
