//**********************************************************************
//COPYRIGHT NOTICE AND DISCLAIMER
//
//Copyright (c) Erik Meijering. All rights reserved.
//
//Permission to copy and use this software and accompanying
//documentation for educational, research, and not-for-profit
//purposes, without fee and without a signed licensing agreement,
//is hereby granted, provided that the above copyright notice, this
//paragraph and the following two paragraphs appear in all copies.
//The copyright holder is free to make upgraded or improved versions
//of the software available for a fee or commercially only.
//
//IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE TO ANY PARTY FOR
//DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
//OF ANY KIND WHATSOEVER, ARISING OUT OF THE USE OF THIS SOFTWARE
//AND ITS DOCUMENTATION, EVEN IF HE HAS BEEN ADVISED OF THE
//POSSIBILITY OF SUCH DAMAGE.
//
//THE COPYRIGHT HOLDER SPECIFICALLY DISCLAIMS ANY WARRANTIES,
//INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
//MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
//SOFTWARE AND ACCOMPANYING DOCUMENTATION IS PROVIDED "AS IS".
//THE COPYRIGHT HOLDER HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
//SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
//**********************************************************************
package neuronJgui;

import ij.IJ;
import ij.gui.GUI;

import java.awt.Button;
import java.awt.Choice;
import java.awt.Component;
import java.awt.Dialog;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Panel;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;

public final class NamesDialog extends Dialog implements ActionListener, FocusListener {

    private final Choice namesChoice;
    private final TextField nameField;

    private final Button okayButton;
    private final Button cancelButton;

    // Builds the dialog for renaming the types and clusters.
    NamesDialog() {

	super(IJ.getInstance(),"NeuronJ: Rename",true);
	final GridBagConstraints c = new GridBagConstraints();
	final GridBagLayout grid = new GridBagLayout();
	setLayout(grid);

	// Add choice:
        namesChoice = new Choice();
	final int nrtypes = NJ.types.length;
        for (int i=1; i<nrtypes; ++i) namesChoice.addItem(NJ.types[i]);
	final int nrclusters = NJ.clusters.length;
        for (int i=1; i<nrclusters; ++i) namesChoice.addItem(NJ.clusters[i]);
        namesChoice.select(0);
	c.gridx = 0; c.gridy++; c.gridwidth = 1;
        c.anchor = GridBagConstraints.EAST;
        c.insets = new Insets(20, 13, 5, 0);
        grid.setConstraints(namesChoice, c);
        add(namesChoice);

	// Add text field:
        c.gridx = 1; c.gridwidth = 2;
        c.anchor = GridBagConstraints.WEST;
        c.insets = new Insets(20, 10, 5, 15);
 	nameField = new TextField("", 15);
        grid.setConstraints(nameField, c);
        nameField.setEditable(true);
	nameField.addFocusListener(this);
	add(nameField);

	// Add Okay and Cancel buttons:
	final Panel buttons = new Panel();
        buttons.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 0));
        okayButton = new Button("   OK   ");
        okayButton.addActionListener(this);
        cancelButton = new Button("Cancel");
        cancelButton.addActionListener(this);
        buttons.add(okayButton);
        buttons.add(cancelButton);
	c.gridx = 0; c.gridy++; c.gridwidth = 3;
        c.anchor = GridBagConstraints.EAST;
        c.insets = new Insets(15, 10, 12, 10);
        grid.setConstraints(buttons, c);
        add(buttons);

	// Pack and show:
	pack();
	GUI.center(this);
	setResizable(false);
	show();
    }

    public void actionPerformed(final ActionEvent e) {

	if (e.getSource() == okayButton) {
	    final String name = nameField.getText();
	    final int index = namesChoice.getSelectedIndex() + 1;
	    if (index < NJ.types.length) NJ.types[index] = name;
	    else NJ.clusters[index-NJ.types.length+1] = name;
	    IJ.showStatus("Changed name");
	    NJ.save = true;
	} else { IJ.showStatus(NJ.COPYRIGHT); }

	setVisible(false);
	dispose();
    }
	
    public void focusGained(final FocusEvent e) {
	Component c = e.getComponent();
	if (c instanceof TextField) ((TextField)c).selectAll();
    }
	
    public void focusLost(final FocusEvent e) {
	Component c = e.getComponent();
	if (c instanceof TextField) ((TextField)c).select(0,0);
    }

}
