package neuronJgui;

// **********************************************************************
// COPYRIGHT NOTICE AND DISCLAIMER
//
// Copyright (c) Erik Meijering. All rights reserved.
//
// Permission to copy and use this software and accompanying
// documentation for educational, research, and not-for-profit
// purposes, without fee and without a signed licensing agreement,
// is hereby granted, provided that the above copyright notice, this
// paragraph and the following two paragraphs appear in all copies.
// The copyright holder is free to make upgraded or improved versions
// of the software available for a fee or commercially only.
//
// IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE TO ANY PARTY FOR
// DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES,
// OF ANY KIND WHATSOEVER, ARISING OUT OF THE USE OF THIS SOFTWARE
// AND ITS DOCUMENTATION, EVEN IF HE HAS BEEN ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
//
// THE COPYRIGHT HOLDER SPECIFICALLY DISCLAIMS ANY WARRANTIES,
// INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
// MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE. THE
// SOFTWARE AND ACCOMPANYING DOCUMENTATION IS PROVIDED "AS IS".
// THE COPYRIGHT HOLDER HAS NO OBLIGATION TO PROVIDE MAINTENANCE,
// SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
//
// **********************************************************************

import ij.IJ;
import ij.plugin.PlugIn;
import java.lang.System;

// Launches NeuronJ and together with its auxiliary classes takes care
// of handling all interactions.
public final class NeuronJ_ implements PlugIn {
	
    // Performs checks and launches the application.
    public void run(final String arg) {

	// Currently it is not possible to have multiple instances of
	// NeuronJ working in parallel:
	if (NJ.ntb != null) {
	    NJ.notify("NeuronJ is already running.");
	    return;
	}

	final long lStartTime = System.currentTimeMillis();
	
	// Before doing anything, load the parameter settings, which
	// also determine whether or not to show debug messages:
	final float paramsversion = NJ.loadParams();
	NJ.write("\nNeuronJ version "+NJ.VERSION);
	NJ.write("Copyright (C) by Erik Meijering");
	if (paramsversion < 0.0f) NJ.write("Unable to load parameters\nUsing default parameter settings");
	else if (paramsversion == NJ.VERSION) NJ.write("Loaded parameters");
	else NJ.write("Loaded parameters from version "+paramsversion);

	// Install NeuronJ toolbar and handler:
	NJ.ntb = new TracingToolbar();
	NJ.nhd = new TracingHandler();

	IJ.showStatus(NJ.COPYRIGHT);
	NJ.write("Initialization completed in "+IJ.d2s((System.currentTimeMillis()-lStartTime)/1000f,2)+" seconds");
    }
}
